/*
** Module   : Session.java
** Abstract : Interface for a network session
**
** Copyright (c) 2004-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- -----------------------------Description-----------------------------
** 001 ECF 20071101  @35860  Created initial version. Interface for a
**                           network session.
** 002 CA  20140902          Added addSessionListener(SessionListener, boolean) to conditionally
**                           register the listener, based on the queue state (running or not), and
**                           also ensuring the queue state is not changed during listener 
**                           registration.
*/

package com.goldencode.p2j.net;

import java.io.*;

/**
 * Defines the public API for a network session, including methods for low
 * level message transport, and methods for higher level session management.
 */
public interface Session
extends LowLevelSession
{
   /**
    * Add a new instance into the list of objects which will be notified
    * of session events.
    * 
    * @param   listener
    *          The listener to add.
    */
   public void addSessionListener(SessionListener listener);

   /**
    * Add a new instance into the list of objects which will be notified
    * of session events.
    * <p>
    * If the <code>check</code> flag is set, then it ensure that the session is still connected
    * before adding the listener.  If the session is not connected, then the listener will not
    * be added and this will return <code>false</code>.
    * 
    * @param   listener
    *          The listener to add.
    * @param   check
    *          Flag to check session state before adding.
    * 
    * @return  <code>true</code> if the listener was added. <code>false</code> otherwise.
    */
   public boolean addSessionListener(SessionListener listener, boolean check);
   
   /**
    * Remove the listener from the list of objects to be notified about
    * session events.
    * 
    * @param   listener
    *          The listener to remove.
    */
   public void removeSessionListener(SessionListener listener);
   
   /**
    * Sets the instance of the state synchronizer the reader and writer
    * threads should use, if and only if there is no other state synchronizer
    * already in use.
    *
    * @param   sync
    *          The synchronizer to register or <code>null</code> if the
    *          currently registered synchronizer should be deregistered.
    */
   public void registerSynchronizer(StateSynchronizer sync);
   
   /**
    * Send provided message as an asynchronous request without waiting for a
    * reply.  The message type will be forced to <code>REQUEST_ASYNCH</code>.
    * 
    * @param   message
    *          Request message.
    */
   public void forward(Message message);
   
   /**
    * Send provided message as a synchronous request and wait for reply.
    * Return message payload as a result.
    * <p>
    * The message passed as a parameter must be completely filled with all
    * required information - routing key should point to valid entry point. If
    * addresses in the routing key are not set then destination is assumed at
    * directly connected node and appropriate addresses are assigned
    * automatically.
    * <p>
    * Caller can specify timeout for waiting reply message. If timeout is set
    * to 0 then queue will wait for reply as long as queue is running.
    * 
    * @param   message
    *          Request message.
    * @param   timeout
    *          Reply wait timeout.
    * 
    * @return  Reply message payload.
    * 
    * @throws  Exception
    *          Is thrown if reply received from remote side has an exception
    *          as its payload, if a timeout occurs before the reply message
    *          is received or if a problem occurs during transmission.
    */
   public Object transact(Message message, int timeout)
   throws Exception;
   
   /**
    * Terminate the network session.  This method should notify all session
    * listeners of the termination event, and will stop the underlying queue
    * if this is the only session using it.
    */
   public void terminate();
   
   /**
    * Send an echo request message using provided data as a message payload
    * and wait for reply. Return message payload as a result.
    * <p>
    * If instance of <code>RoutingKey</code> is provided then message is
    * sent to node pointed by the <code>RoutingKey</code>. Otherwise
    * directly connected node is assumed. Note that processing of the ECHO
    * message ignores entry point information in <code>RoutingKey</code> so
    * an instance of <b>RoutingKey </b> created with default constructor can
    * be used.
    * <p>
    * Caller can specify timeout for waiting reply message. If this is not
    * necessary then 0 should be passed as If timeout is set to 0 then queue
    * will wait for reply as long as queue is running.
    * 
    * @param    obj
    *           Message payload.
    * @param    timeout
    *           Reply wait timeout.
    * @param    key
    *           The destination node address information.
    * 
    * @return   Payload of the echo reply.
    * 
    * @throws   InterruptedException
    *           forwarded from transactImpl()
    * @throws   RequestTimeoutException
    *           forwarded from transactImpl()
    * @throws   ProtocolViolation
    *           in case of protocol errors.
    * @throws   Exception
    *           Is thrown if a timeout occurs before the reply message
    *           is received or if a problem occurs during transmission.
    */
   public Object echo(Serializable obj, int timeout, RoutingKey key)
   throws ProtocolViolation,
          InterruptedException,
          RequestTimeoutException,
          Exception;
}
